from abc import ABC
from typing import Optional

from kilroy_face_py_shared.models import SerializableModel
from pydantic import root_validator


class TextData(SerializableModel):
    content: str


class ImageData(SerializableModel):
    raw: str
    filename: str


class BasePost(SerializableModel, ABC):
    pass


class TextOnlyPost(BasePost):
    text: TextData


class ImageOnlyPost(BasePost):
    image: ImageData


class TextAndImagePost(BasePost):
    text: TextData
    image: ImageData


class TextOrImagePost(BasePost):
    text: Optional[TextData] = None
    image: Optional[ImageData] = None

    @root_validator(pre=True)
    def check_if_at_least_one_present(cls, values):
        if "text" not in values and "image" not in values:
            raise ValueError("Any of text or image is required.")
        return values

    class Config:
        schema_extra = {
            "anyOf": [
                {"required": ["text"]},
                {"required": ["image"]},
            ]
        }


class TextWithOptionalImagePost(BasePost):
    text: TextData
    image: Optional[ImageData] = None


class ImageWithOptionalTextPost(BasePost):
    text: Optional[TextData] = None
    image: ImageData



from kilroy_face_py_shared.errors import (
    AppError,
    INTERNAL_SERVER_ERROR,
    INVALID_CONFIG_ERROR,
    PARAMETER_GET_ERROR,
    PARAMETER_SET_ERROR,
    STATE_NOT_READY_ERROR,
)
from kilroy_face_py_shared.messages import (
    GeneratedPost,
    GetConfigRequest,
    GetConfigResponse,
    GetConfigSchemaRequest,
    GetConfigSchemaResponse,
    GetMetadataRequest,
    GetMetadataResponse,
    GetPostSchemaRequest,
    GetPostSchemaResponse,
    GetStatusRequest,
    GetStatusResponse,
    PostRequest,
    PostResponse,
    RealPost,
    ScoreRequest,
    ScoreResponse,
    ScrapRequest,
    ScrapResponse,
    SetConfigRequest,
    SetConfigResponse,
    Status,
    WatchConfigRequest,
    WatchConfigResponse,
    WatchStatusRequest,
    WatchStatusResponse,
    ResetRequest,
    ResetResponse,
)
from kilroy_face_py_shared.metadata import Metadata
from kilroy_face_py_shared.models import SerializableModel
from kilroy_face_py_shared.posts import (
    BasePost,
    ImageData,
    ImageOnlyPost,
    ImageWithOptionalTextPost,
    TextAndImagePost,
    TextData,
    TextOnlyPost,
    TextOrImagePost,
    TextWithOptionalImagePost,
)
from kilroy_face_py_shared.resources import (
    resource,
    resource_bytes,
    resource_text,
)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['kilroy_face_py_shared', 'kilroy_face_py_shared.resources']

package_data = \
{'': ['*']}

install_requires = \
['betterproto>=2.0.0b5,<3.0.0',
 'jsonschema>=4.7,<5.0',
 'pydantic>=1.9,<2.0',
 'pyhumps>=3.7,<4.0']

setup_kwargs = {
    'name': 'kilroy-face-py-shared',
    'version': '0.5.1',
    'description': 'shared code for kilroy face SDKs in Python 🤝',
    'long_description': '<h1 align="center">kilroy-face-py-shared</h1>\n\n<div align="center">\n\nshared code for kilroy face SDKs in Python 🤝\n\n[![Lint](https://github.com/kilroybot/kilroy-face-py-shared/actions/workflows/lint.yaml/badge.svg)](https://github.com/kilroybot/kilroy-face-py-shared/actions/workflows/lint.yaml)\n[![Tests](https://github.com/kilroybot/kilroy-face-py-shared/actions/workflows/test-multiplatform.yaml/badge.svg)](https://github.com/kilroybot/kilroy-face-py-shared/actions/workflows/test-multiplatform.yaml)\n[![Docs](https://github.com/kilroybot/kilroy-face-py-shared/actions/workflows/docs.yaml/badge.svg)](https://github.com/kilroybot/kilroy-face-py-shared/actions/workflows/docs.yaml)\n\n</div>\n\n---\n\nTODO\n\n## Installing\n\nUsing `pip`:\n\n```sh\npip install kilroy-face-py-shared\n```\n',
    'author': 'kilroy',
    'author_email': 'kilroymail@pm.me',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kilroybot/kilroy-face-py-shared',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass
from datetime import datetime
from typing import Optional

import betterproto


class Status(betterproto.Enum):
    STATUS_UNSPECIFIED = 0
    STATUS_LOADING = 1
    STATUS_READY = 2


@dataclass(eq=False, repr=False)
class GetMetadataRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetMetadataResponse(betterproto.Message):
    key: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GetPostSchemaRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetPostSchemaResponse(betterproto.Message):
    schema: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetStatusRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetStatusResponse(betterproto.Message):
    status: "Status" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class WatchStatusRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class WatchStatusResponse(betterproto.Message):
    status: "Status" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class GetConfigSchemaRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetConfigSchemaResponse(betterproto.Message):
    schema: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetConfigRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetConfigResponse(betterproto.Message):
    config: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class WatchConfigRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class WatchConfigResponse(betterproto.Message):
    config: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class SetConfigRequest(betterproto.Message):
    config: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class SetConfigResponse(betterproto.Message):
    config: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GeneratedPost(betterproto.Message):
    content: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class PostRequest(betterproto.Message):
    post: "GeneratedPost" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PostResponse(betterproto.Message):
    post_id: str = betterproto.string_field(1)
    post_url: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ScoreRequest(betterproto.Message):
    post_id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ScoreResponse(betterproto.Message):
    score: float = betterproto.double_field(1)


@dataclass(eq=False, repr=False)
class ScrapRequest(betterproto.Message):
    limit: Optional[int] = betterproto.uint64_field(
        1, optional=True, group="_limit"
    )
    before: Optional[datetime] = betterproto.message_field(
        2, optional=True, group="_before"
    )
    after: Optional[datetime] = betterproto.message_field(
        3, optional=True, group="_after"
    )


@dataclass(eq=False, repr=False)
class RealPost(betterproto.Message):
    id: str = betterproto.string_field(1)
    content: str = betterproto.string_field(2)
    score: float = betterproto.double_field(3)


@dataclass(eq=False, repr=False)
class ScrapResponse(betterproto.Message):
    post: "RealPost" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ResetRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ResetResponse(betterproto.Message):
    pass



class AppError(Exception):
    def __init__(self, code, reason):
        self.code = code
        self.reason = reason
        super().__init__(self.reason)

    def __str__(self):
        return f"{self.code}: {self.reason}"


INTERNAL_SERVER_ERROR = AppError(-1, "Internal server error.")
PARAMETER_GET_ERROR = AppError(1, "Unable to get parameter value.")
PARAMETER_SET_ERROR = AppError(2, "Unable to set parameter value.")
STATE_NOT_READY_ERROR = AppError(3, "State is not ready to be read.")
INVALID_CONFIG_ERROR = AppError(4, "Config is not valid.")
